# -*- coding: utf-8 -*-

__author__ = 'Learning Equality'
__email__ = 'jordan@learningequality.org'
__version__ = '0.6.17'


import sys

if sys.version_info < (3, 4, 0):
    raise RuntimeError("Ricecooker only supports Python 3.4+")
